/** 
@(#) AboutBox.java	
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

import brr.awt.TextPanel;
import brr.awt.ImagePanel;
import brr.awt.FlyButton;
import brr.awt.MultiLine;
import java.awt.*;

/**

AboutBox constructs a Frame with an Image, a head line and some text. 
The Frame is invisible when instantiated. To display the AboutBox, just
call it's show() method. The AboutBox is displayed and brought to the front.
When the OK button is clicked, the AboutBox is hidden.<p>

<strong>Beans Migration Notes:</strong><p>
<ul>
<li>The show() method is deprecated. Use setVisible(true) instead.<p>
</ul>
<p><applet code=brr.awt.examples.AboutBoxExample.class width=200 height=50>
</applet>
<p>
Source: <a href="brr/awt/examples/AboutBoxExample.java">AboutBoxExample.java</a><p>
*/

public class AboutBox extends java.awt.Frame {
    Panel header, side, down;

    /** This sets the default Font for the head line. */
    public static Font DEFAULT_HEADLINE_FONT = 
		new Font("Helvetica", Font.BOLD, 20);

    Image OKButtonImage, aboutImage;

    /** Creates an AboutBox with the DEFAULT_HEADLINE_FONT.
	The MultiLine is created with it's default settings.
	@see TextPanel
	@see MultiLine
	@see java.awt.Image
	@see java.lang.String
    */
    public AboutBox(String headLine, String aboutInfo, 
		    Image aboutImage, 
		    Image OKButtonImage,
		    Image iconImage) {
	this(   new TextPanel(headLine, DEFAULT_HEADLINE_FONT), 
		new MultiLine(aboutInfo),
		aboutImage, OKButtonImage, iconImage
	);
    }
   
    /** This constructor gives complete control over the look of the AboutBox.
	@see TextPanel
	@see MultiLine
	@see java.awt.Image
    */
    public AboutBox(TextPanel str, MultiLine mul, 
		    Image aboutImage,
		    Image OKButtonImage,
		    Image iconImage) {

	this.aboutImage = aboutImage;
	this.OKButtonImage = OKButtonImage;

	setLayout(new BorderLayout());

	add("West", side = new Panel());
	side.add(new ImagePanel(aboutImage, false));
	add("North", header = new Panel());
	//header.add(new ImagePanel(aboutImage, false));
	header.add(str);

	add("Center", mul);

	add("South", down = new Panel());
	//down.setLayout(new FlowLayout());
	down.add(new FlyButton(OKButtonImage, "OK", false));

	if (iconImage != null) setIconImage(iconImage);

	pack();
    }

    public boolean action(Event e, Object arg) {
	if (e.target instanceof FlyButton) {
	    this.hide();
	}
	return false;
    }

    /** Displays the AboutBox and brings it to the front.
	This method is deprecated. Use setVisible(true) instead.
	@deprecated
    */
    public void show() { setVisible(true); }

    /** Displays the AboutBox and brings it to the front.
    */
    public void setVisible(boolean visible) {
	if (visible) {
	    super.show();
	    toFront();
	} else {
	    super.hide();
	}
    }
};
