/** @(#)Align.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

/**
    This class is used for defining alignment of a "component" inside 
	a "container".<p>
	The terms "component" and "container" are used loosely here and
	do not necessarily have to be java.awt.Component and java.awt.Container.
	Here "component" is the entity that is aligned inside the "container"<p>
<applet code=brr.awt.examples.AlignExample.class width = 200 height = 100>
</applet>
<p>Source: <a href="brr/awt/examples/AlignExample.java">AlignExample.java</a><p>

*/

public class Align {
    /** center the text */
    public static final int CENTER 	= 0;
    /** center the text (British spelling)*/
    public static final int CENTRE 	= 0;	// for the british
    /** Align the text to the left*/
    public static final int LEFT 	= 1;
    /** Align the text to the right*/
    public static final int RIGHT 	= 2;
    /** Justify the text along both margins*/
    public static final int BOTH 	= 3;
    /** Justify the text along both margins*/
    public static final int JUSTIFY 	= 3;

    /** Calculate the # pixels on the left to align a component in a container
	or or text string in a Canvas/MultiLine/TextPanel etc..<p>

	@param align the alignment of the component.<p>
	@param length the # pixels the occupied by the component.<p>
	@param width the width of the container in pixels.<p>

	Example:<p>
	<pre>
	FontMetrics fm = getFontMetrics(this.getFont());
	Align.adjustAlign(Align.LEFT, fm.stringWidth(str), 200);
	</pre>
    */
    public static int adjustAlign(int align, int length, int width) {
        if ( align == RIGHT )     return width - length-1;
        if ( align == CENTER )    return (width - length)/2;
        if ( align == LEFT )      return  0;
        // default: act like LEFT align
        return 0;
    }

};
