/** @(#)MultiLine.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.awt;

import java.awt.*;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Vector;
import brr.awt.Align;

class Line extends Object {
    String text = null;
    int x = 0, y = 0;
    public Line(String text, int x, int y) {
	this.text = text; this.x = x; this.y = y;
    }
};

/**
MultiLine acts like a multi-line label. This is great for displaying multiple
lines of text without having to resort to multiple java.awt.Label or 
brr.awt.TextPanel objects.<p>
<applet code=brr.awt.examples.MultiLineExample.class height=100 width=200>
</applet>
<p>Source: <a href="brr/awt/examples/MultiLineExample.java">MultiLineExample.java</a><p>

*/
public class MultiLine extends java.awt.Canvas {
    String text = null;
    int y = 0;
    int width, height ;
    static final int DEFAULT_ALIGN = Align.LEFT;
    int align = DEFAULT_ALIGN;
    Vector lineVector = new Vector();
    static final int DEFAULT_CHARS_PER_LINE = 25;
    int charsPerLine = DEFAULT_CHARS_PER_LINE;
    boolean clip = false;
    String DEFAULT_WIDTH_STRING = "1234567890123456789012345";

    /** Creates a MultiLine object with 25 characters per line.
	@param text the initial text
    */
    public MultiLine(String text) {
	this(text, DEFAULT_CHARS_PER_LINE);
    }

    /** Creates a MultiLine object with the specified # characters per line.
	<p>When the text or Font is changed, the MultiLine resizes.
	@param text the initial text
	@param charsPerLine the number of characters per line. 
    */
    public MultiLine(String text, int charsPerLine) {
	this.charsPerLine = charsPerLine;
	this.align = align;
	this.text = text;
	clip = false;
	setFont(new Font("Helvetica", Font.BOLD, 12));
	formatText();
	//resize(width, height);
	setText(text);
    }

    /**Explicit constructor: Creates a MultiLine object with the specified 
	width and height in pixels.
	<p>When the text or Font is changed, the MultiLine does not resize.
        @param text the initial text
        @param charsPerLine the number of characters per line. 
    */
    public MultiLine(String text, int width, int height) {
	clip = true;
	resize(this.width = width, this.height = height);
	setText(text);
    }

    /** Set the font to render the MultiLine. Will resize unless explicit
	constructor was used. 
    */
    public void setFont(Font font) { 
	super.setFont(font); formatText(); repaint(); 
    }

    /** Set the text of the MultiLine. Will resize unless explicit 
        constructor was used. 
    */
    public void setText(String text) { this.text = text; repaint(); }

    /** Set the alignment of the MultiLine text. 
    */
    public void setAlign(int align) { this.align = align; repaint(); }

    /** @return the MultiLine text. 
    */
    public String getText() { return text; }

    /** @return the alignment of the MultiLine text. 
    */
    public int getAlign() { return align;  }

    /** called by setText() or setFont() to resize the MultiLine.
	Not normally called by other classes.
    */
    public void formatText() {
	lineVector.removeAllElements();
	if (text != null) {
	    FontMetrics fm = getFontMetrics(this.getFont()); 
	    width = clip ? size().width : (
			    text.length() < charsPerLine? 
	    		    fm.stringWidth(DEFAULT_WIDTH_STRING) : 
	    		    fm.stringWidth(text.substring(0,charsPerLine - 1))
			);
	    y = fm.getMaxAscent(); // first line's base line
	    int spaceWidth = fm.charWidth(' ');
	    int x = 0; // start leftmost
	    String line = "";
	    StringTokenizer st = new StringTokenizer(text, " \n\t", true);

	    while(st.hasMoreTokens()) {
		String word = st.nextToken();
		if ( word.equals(" ") || word.equals("\t") ) continue;
		int wordWidth = fm.stringWidth(word);
		if ( wordWidth + x > width || word.equals("\n") ) { 
		    lineVector.addElement(new 
		    	Line(line, Align.adjustAlign(align, x, width), y)
		    );
		    y += fm.getHeight(); x = 0; 
		    line = "";
		} 
		if ( !word.equals("\n")) {
		x += wordWidth + spaceWidth;
		line = line + word + " ";
		}
	    }
	    // draw the last line
	    lineVector.addElement(
		new Line(line, Align.adjustAlign(align, x, width), y)
		);
	    height = clip? height : y+fm.getMaxDescent();
	    if (!clip) resize(width, height);
	}
    }

    public void paint(Graphics g) {
	formatText();
	Enumeration strings = lineVector.elements();
	while(strings.hasMoreElements()) {
	    Line line = (Line)strings.nextElement();
	    g.drawString(line.text, line.x, line.y);
	}
    }
};
