/** @(#)ProgressBar.java	
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/
    
package brr.awt;

import brr.awt.*;
import java.awt.*;

/**
Displays a progress bar. This can be subclassed to change just the display.
For example, to make it an odometer or fuel guage.<p>
<applet code=brr.awt.examples.ProgressBarExample.class width = 200 height = 50>
</applet>
<applet code=brr.awt.examples.ProgressBarExample.class width = 200 height = 50>
<param name=style value=0>
</applet>
<p>Source: <a href="brr/awt/examples/ProgressBarExample.java">ProgressBarExample.java</a><p>

*/
public class ProgressBar extends java.awt.Canvas {
    int percent = 0;
    int borderSize = 2;
    Color doneColor;
    Color toBeDoneColor;

    boolean toggle = true;

    /** Creates a ProgressBar with the specified width and height.
	Uses the default background color of lightGray, default stuff to be 
	done color of white and the default stuff done color of blue.
	@param width width of the ProgressBar.
	@param height height of the ProgressBar.
    */
    public ProgressBar(int width, int height) {
	this(width, height, Color.blue, null);
    }

    /** Creates a ProgressBar with the specified width and height and custom 
	colors.
        @param width width of the ProgressBar.
        @param height height of the ProgressBar.
	@param doneColor This color indicates the amount of work that is done.
	@param toBeDoneColor This color indicates the amount of work to be done.
		Set this to null to show the default background color.
    */
    public ProgressBar(int width, int height, 
		 Color doneColor, Color toBeDoneColor){
	this.doneColor = doneColor;
	this.toBeDoneColor = toBeDoneColor;
	resize(width, height);
    }

    /** @return the size of the 3D border.
    */
    public int getBorderSize() { return borderSize; }
 
    /** Sets the size of the 3D border.
        @param borderSize the width of the new border in pixels.
    */
    public void setBorderSize(int borderSize) {
        this.borderSize = borderSize;
        repaint();
    }

    /** @return the percent work already done.
    */
    public int getPercent() { return percent; }

    /** Sets the work already done.
	@param percent percent work finished. The ProgressBar updates the
	display to reflect this new value.
    */
    public void setPercent(int percent) { 
	if(percent < this.percent) toggle = true;
	this.percent = percent; 
	repaint(); 
    }

    public void update(Graphics g) {
        int inset = 2*borderSize+1;

	if ( toggle) { 
	    paint(g); 
	    toggle = false; 
	} else {
	    g.setColor(doneColor);
        g.fillRect(borderSize, borderSize,
                (size().width-inset)*percent/100, size().height-inset);
	}
    }

    public void paint(Graphics g) {
	int inset = 2*borderSize+1;

	g.setColor(getBackground());
	for(int i = 0; i < borderSize; i++)
	    g.draw3DRect(i,i, size().width-(2*i+1),size().height-(2*i+1),false);
	g.setColor(toBeDoneColor != null ? toBeDoneColor : getBackground());
	g.fillRect(borderSize, borderSize, 
		size().width-inset, size().height-inset);
	g.setColor(doneColor);
	g.fillRect(borderSize, borderSize, 
		(size().width-inset)*percent/100, size().height-inset);
    }
};
	
