/*     Talk.java  -- A simple Talk program
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)
*/

/*
  This is a modification of my first Java networking program. The code is 
  not exactly what you would read with your grandmother without getting 
  embarassed, but it does show how to use Pipes.


         Host 1                                  Host 2

       []---------[]                          []-----------[]
        | server  |  <-----------------------  | client    |
       []---------[]                          []-----------[]

       []---------[]                          []-----------[]
        |  client |  ----------------------->  | server    |
       []---------[]                          []-----------[]

*/
package brr.io.examples;
import java.awt.*;
import java.io.*;
import brr.io.Pipe;
import java.net.*;

/** TalkServer waits till it gets the TALK_PORT and then waits for a connection
from the remote client. Then it makes sure it's the right client by reading 
names of clients and matching them with it's own. If everything is fine, it
closes the ServerSocket else it terminates the connection and continues to wait.
Uses just one Talk port, so you cant talk to someone else on the same machine.
This can be rectified by using more than one port and trying to get one of them.
The TalkClient has to try connecting to all ports on target machine.
*/

class TalkServer extends Thread {
    public static final int TALK_PORT = 6666;
    public boolean busy = true;
    public boolean waiting = true;
    boolean found = false;
    ServerSocket ss;Socket s;
    connect c;
    public TalkServer(connect conn) throws IOException {
	c = conn;
    } 
  
    public void run() { 
	System.out.println("Waiting for connection Request");
	try {
	    // First try to get Server socket. Someone else may be using it.
	    do {
		busy = false;
		try {
		ss = new ServerSocket(TALK_PORT);
		} catch(SocketException e) { busy = true; }
	    } while( busy) ;
	    // Ok, we got it. Now accept connections
	    do {
		s = ss.accept();
		c.in = new DataInputStream(s.getInputStream());
		// read who it is and whom they want to talk to
		String line = c.in.readLine();
		// is it the right person and does he want to talk to us?
		if ( line.equalsIgnoreCase(c.otheruser+c.localuser)) 
		found = true;	// yes, exit loop
		else
		s.close();	// no terminate connection
	    } while (!found); 
	    // ready to receive from other end
	    System.out.println("Connection Accepted.");
	    ss.close();
	} catch(IOException io) { 
	    System.err.println("Socket IO Exception"); 
	    io.printStackTrace();
	}
	waiting = false;
    }
};

/** 
   TalkClient tries to connect to the remote server and establish connection
   */

class TalkClient extends Thread {
    public boolean waiting = true;
    connect c;
    Socket s;
    public TalkClient(connect conn) {
	c = conn;
    }
    public void run() {
	boolean conn = false;
	System.out.println("Connecting to: "+c.remotehost);
	do {
	// repeatedly try to connect. Have to try until the remote SocketServer
	// is actually ready and accepting. So ignore exceptions till then
	    conn = true;
	    try {
	    s = new Socket(c.remotehost, 6666);
	    } catch (IOException e) { conn = false;}
	    try { Thread.sleep(100); } catch(InterruptedException e) {}
	} while ( conn == false);
	try { 
	    // we got it. Write our name and the party we want to talk to
	    c.out = new DataOutputStream(s.getOutputStream());
	    c.out.writeBytes(c.localuser+c.otheruser+"\n");
	} catch (IOException e) { }
	// ready to talk
	waiting = false;
    }
};

class connect
{
    public final int TALK_PORT = 6666;
    public String otheruser;
    public String localuser;
    public String remotehost;
    public DataInputStream in;
    public DataOutputStream out;
    public connect(String args[]) 
        throws IOException, UnknownHostException, InterruptedException {
	otheruser = args[0].substring(0, args[0].indexOf('@'));
	remotehost = args[0].substring(args[0].indexOf('@')+1);
	try { 
	    localuser = args[1]; 
	} catch( ArrayIndexOutOfBoundsException e) {
	    localuser = System.getProperty("user.name");
	}
	System.out.println("Attempting connection to "+otheruser+" at "+remotehost);
	TalkServer gp = new TalkServer(this); gp.start();
	do { Thread.sleep(100); } while(gp.busy);
	TalkClient gc = new TalkClient(this); gc.start();
	do { Thread.sleep(100); } while(gp.waiting || gc.waiting);
	System.out.println("Connection Established: "+args[0]);
    }
};



public class Talk 
{
    public static void main(String args[]) 
        throws IOException, UnknownHostException, InterruptedException {
	if (args[0] == null) {
	    System.out.println("usage: talk user@host");
	    System.exit(1);
	}
	connect c = new connect(args);
	(new Pipe(System.in, c.out)).start();
	(new Pipe(c.in, System.out)).start();
    }
};
