/** @(#)Loadable.java  
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/
package brr.net;

/** Interface for all classes that can be loaded over the network. 
<p>
Normally, the following implementations can be used verbatim:
<pre>
Object loader;

public void setLoader(Object loader) {
    this.loader = loader;  // save
}

public Applet getLoader() {
    return loader;  
}
</pre>
Example:<p>
<pre>
public class SomeGame extends Applet implements brr.net.Loadable {
...
    public SomeGame() { // Constructor
    // bare minimum constructor here.
	....
    }
    
    // this is called after the constructor
    public void setLoader(Object loader) {
	this.loader = loader;  // save 
    }

    public void init() {
	// Put the rest of the constructor here.
	// This is called after setLoader.
	// So you can access parent's functions here.
       name = ((Applet)loader).getParameter("name");  // if parent is an Applet.
       game = getLoader().getParameter("game");  // same as above
	....
    }

    public Applet getLoader() {
    	return loader;  
    }
....
}
</pre>
<p>
<applet code=brr.applet.LoaderApplet.class height=50 width=100>
<param name=class value=brr.net.examples.LoadableExample>
<param name=message value="Click Me">
<param name=image value=images/green-ball.gif>
</applet>
<p>Source: <a href="brr/net/examples/LoadableExample.java">LoadableExample.java</a><p>

*/

public interface Loadable {
    /** @return the parent Applet.
    */
    public Object getLoader();

    /** Called by the loading Object to set the Parent applet value.
    */
    public void setLoader(Object loader);

    /** Called after setLoader by the loading Object.
        <p>Since the setLoader method cannot be called
        before the constructor is called, the statements which need to access
        the socket and applet variables set by setLoader
        must be placed in init. <p>
        A Loader generally follows the following sequence:<p>
        <ol>
        <li> Load the Loadable class if necessary
        <li> Create a new Instance // this calls the constructor
        <li> call setLoader
        <li> call init();
        </ol>
        <p> For all practical cases, put the bulk of the statements in init().
        <p> Since loaders usually call the default constructor, unless you
        specifically need to do something before setLoader
        are called, you may simply use init() as the Constuctor.
    */

    public void init();
};

