/** @(#)Networkable.java      
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.net;

import java.net.Socket;
import brr.net.Loadable;

/** To collaborate an applet (example: play a network game,)
    this interface must be implemented.
    The loading Object constructs a new Instance of the loaded Object
    and calls the setSocket, setLoader and the init methods.<p>

    The collaborating applet must place the statements normally placed in the 
    constructor in the init method if the setSocket and setLoader 
    methods are required to proceed with the game. Example: to communicate
    over the network or call a getParameter method.<p>

Typically, the following implementations can be used verbatim:<p>
<pre>

Socket socket;

public void setSocket(Socket socket) {
    this.socket = socket;
}

public Socket getSocket() {
    return socket;
}
</pre>
You must also implement Loadable, which this interface extends.
<p>
@see Loadable

*/ 

public interface Networkable extends Loadable {
    /** @return the socket set by the loading object.
    */ 
    public Socket getSocket();

    /** Called by the loading Object to set the socket connected to
  	the Game Server. 
    */
    public void setSocket(Socket s);

    /**Called by the loading Object after calling setSocket and setLoader.
        <p>Since the setSocket and setLoader method cannot be called
        before the constructor is called, the statements which need to access
        the socket and applet variables set by setSocket and setLoader
        must be placed in init. <p>
        A Networkable Loader generally follows the following sequence:<p>
        <ol>
        <li> Load the Networkable if necessary
        <li> Create a new Instance // this calls the constructor
        <li> call setSocket
        <li> call setLoader
        <li> call init();
        </ol>
        <p> For all practical cases, put the bulk of the statements in init().
        <p> Since loaders usually call the default constructor, unless you
        specifically need to do something before setSocket and setLoader
        are called, you may simply use init() as the Constuctor.
    */
    public void init();

    //public void paint(Graphics g);
};

