/** @(#) ProtocolHandler.java	1997 Jul 29 
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.net;

import java.io.*;
import java.lang.Thread;
import brr.net.ProtocolHandler;
import brr.util.Debug;


/**
    This class is the request handler for requests to a server or collaboratory
    application. This class is the "what to be done" which has to be implemented
    when using the <a href="brr.net.Protocol.html">Protocol</a> class. Simply 
    subclass this and implement the run() method.<p>

    Check out a simple example program 
    <a href="brr/net/examples/ProtocolExample.java">ProtocolExample.java</a><P>

    Check out how to use this class to build a simple Web Server:
    <a href="brr/net/examples/GetServer.java">GetServer.java</a><br>
    (Really simple! Only GET, no cgi, no servlets etc.)<p>

*/
public class ProtocolHandler extends Thread implements Cloneable {

    Debug d = new Debug();
    DataInputStream dis;
    DataOutputStream dos;
    PrintStream out;

    InputStream is;
    OutputStream os;
    String request;
    String requestType;
    Protocol protocol;

    /** Sets the InputStream to read from.
    */
    public void setInputStream(InputStream is) {
	this.is = is;
	dis = new DataInputStream(is);
    }

    /** Sets the OutputStream to write to.
    */
    public void setOutputStream(OutputStream os) {
	this.os = os;
	dos = new DataOutputStream(os);
	out = new PrintStream(os);
    }

    /** @return the calling Protocol.
    */
    public Protocol getProtocol () { return protocol ; }

    /** Sets the calling Protocol.
    */
    public void setProtocol (Protocol protocol) { this.protocol  = protocol ; }

    /** Sets the request Type.
    */
    public void setRequestType (String requestType) { 
	this.requestType  = requestType ;
    }

    /** Sets the request string.
    */
    public void setRequest(String request) { this.request = request;}

    /** @return the InputStream to read from.
    */
    public InputStream getInputStream() { return is; }

    /** @return the OutputStream to write to.
    */
    public OutputStream getOutputStream() { return os; }

    /** @return the request string.
    */
    public String getRequest() { return request;}

    /** @return the request type string.
    */
    public String getRequestType () { return requestType ;}

    /** @exception CloneNotSupportedException if clone fails.
    */
    public Object clone() 
	throws CloneNotSupportedException { return super.clone(); }

    /** @return the DataInputStream associated with this instance.
    */
    public DataInputStream getDataInputStream() { return dis; }

    /** @return the DataOutputStream associated with this instance.
    */
    public DataOutputStream getDataOutputStream() { return dos; }

    /** @return the PrintStream associated with this instance.
    */
    public PrintStream getPrintStream() { return out; }

    public void run() { d.println("Null.run()"); }
};
