/** @(#)Debug.java
  @author Bharath Raj Rao
          brr@cat.syr.edu
 
Permission is granted to copy and modify this file as long as this message
including my name and email address is preserved.
 
No warranties. Use at your own risk. (Hey! we live in a litigious world!)

*/

package brr.util;

/** Class for debugging. Use this to programatically control debugging.
    For example, if a variable value exceeds legal bounds...
	
<p> Use as below:<p>
<pre>
    Debug d = new Debug();
    d.println("Hello");      // Will print
    // ... some code
    d.setOn(i > 7);
    d.println("i exceeds 7. Current value:"+i);   // Conditional print
    d.setOn(false);
    callSomeFunc();
    d.println("yuk!!");      // Will print if triggered in function
</pre>
*/
public class Debug {
    boolean on = true;

    /** @return true if debugging is enabled.
    */
    public boolean getOn() { return on; }

    /** Switches debugging on/off.
	@param on if true, debugging is switched on, else switched off.
    */
    public void setOn(boolean on) { this.on = on; }

    /** Prints the object to System.out and adds a newline.
    	@param obj the object to be printed.
    */
    public void println(Object obj) {
	if(on) {
	    System.out.println(""+obj);
	}
    }

    /** Prints the object to System.out.
    	@param obj the object to be printed.
    */
    public void print(Object obj) {
	if(on) {
	    System.out.print(""+obj);
	}
    }
   
    /** Delays for some milliseconds. Does not throw InterruptedException.
    */
    public static void sleep(int n) { 
	try { Thread.sleep(n); } catch(Exception e) { }
    }
};
